@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import base64
 import json
 import pytest
 import queue
@@ -773,3 +774,45 @@ class TestResults:
         assert response.status_code == 200
         assert get.call_count == 3
         assert isinstance(result, type(()))
+
+
+class TestHandlePatch:
+    def test_handle_patch_unknown(self):
+        with pytest.raises(Exception) as excinfo:
+            tuxsuite.build.handle_patch("/tmp/patch")
+        assert "series file not found in patch directory" in str(excinfo)
+
+    def test_handle_patch_mbox(self, mocker, sample_patch, sample_patch_file):
+        guess = mocker.patch(
+            "mimetypes.guess_type", return_value=("application/mbox", None)
+        )
+        result = tuxsuite.build.handle_patch(sample_patch_file)
+        assert guess.call_count == 1
+        assert isinstance(result, type(()))
+        assert isinstance(type(result[0]), type(str))
+        assert base64.b64decode(result[1]) == sample_patch.encode("utf-8")
+
+    def test_handle_patch_dir(self, sample_patch_dir):
+        result = tuxsuite.build.handle_patch(sample_patch_dir)
+        assert isinstance(result, type(()))
+        assert isinstance(type(result[0]), type(str))
+        assert isinstance(type(result[1]), type(str))
+
+    def test_handle_patch_tgz(self, mocker, sample_patch_tgz):
+        result = tuxsuite.build.handle_patch(sample_patch_tgz)
+        assert isinstance(result, type(()))
+        assert isinstance(type(result[0]), type(str))
+        assert isinstance(type(result[1]), type(str))
+
+    def test_handle_patch_tgz_no_series(self, mocker, sample_patch_tgz_no_series):
+        with pytest.raises(Exception) as excinfo:
+            tuxsuite.build.handle_patch(sample_patch_tgz_no_series)
+        assert "series file missing in patch archive" in str(excinfo)
+
+    def test_handle_patch_url(self):
+        url = "https://test.example.com/url"
+        result = tuxsuite.build.handle_patch(url)
+        assert isinstance(result, type(()))
+        assert isinstance(type(result[0]), type(str))
+        assert result[0] == url
+        assert base64.b64decode(result[1]) == b""
