@@ -669,53 +669,32 @@ def test_plan_cli_options(mocker, plan_config, tuxsuite_config):
     plan_obj = tuxsuite.Plan(
         "", token="token", tuxapi_url="http://api", project="prj", group="grp"
     )
-    plan_obj.builds = [mocker.Mock(uid="1"), mocker.Mock(uid="2")]
+    plan_obj.plan = "1sMUTVjE25tQvDmjJ8SjWblpOJr"
+    plan_obj.builds = [
+        mocker.Mock(
+            uid="1",
+            target_arch="arm64",
+            toolchain="gcc-10",
+            status=sample_plan_result["builds"]["1"],
+        ),
+        mocker.Mock(
+            uid="2",
+            target_arch="arm64",
+            toolchain="gcc-11",
+            status=sample_plan_result["builds"]["2"],
+        ),
+    ]
     plan_obj.tests = [
-        mocker.Mock(uid="1", wait_for="1"),
-        mocker.Mock(uid="2", wait_for=None),
+        mocker.Mock(
+            uid="1",
+            wait_for="1",
+            tests=["boot", "ltp-smoke"],
+            status={**sample_plan_result["tests"]["1"], "waiting_for": "1"},
+        ),
     ]
     plan_obj.submit = mocker.Mock()
     plan = mocker.patch("tuxsuite.Plan")
     plan.return_value = plan_obj
-    plan_obj.results = {
-        "builds": {
-            "1": tuxsuite.build.BuildState(
-                build=None,
-                state="finished",
-                status="",
-                message="build 1 pass",
-                icon="🎉",
-                cli_color="green",
-            ),
-            "2": tuxsuite.build.BuildState(
-                build=None,
-                state="finished",
-                status="",
-                message="build 2 pass",
-                icon="🎉",
-                cli_color="green",
-            ),
-        },
-        "tests": {
-            "1": tuxsuite.build.BuildState(
-                build=None,
-                state="finished",
-                status="",
-                message="test 1 pass",
-                icon="🎉",
-                cli_color="green",
-            ),
-            "2": tuxsuite.build.BuildState(
-                build=None,
-                state="finished",
-                status="",
-                message="test 2 pass",
-                icon="🎉",
-                cli_color="green",
-            ),
-        },
-    }
-
     show_log = mocker.patch("tuxsuite.cli.show_log")
     wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
     runner = CliRunner()
@@ -737,12 +716,14 @@ def test_plan_cli_options(mocker, plan_config, tuxsuite_config):
     assert show_log.call_count == 2
 
     assert (
-        """
-Summary: http://api/v1/groups/grp/projects/prj
-* 🎉 build 1 pass: None
-  -> 🎉 test 1 pass: None
-* 🎉 build 2 pass: None
-* 🎉 test 2 pass: None"""
+        """Running Linux Kernel plan 'Simple plan': 'A simple plan'
+Plan http://api/v1/groups/grp/projects/prj/plans/1sMUTVjE25tQvDmjJ8SjWblpOJr
+
+uid: 1sMUTVjE25tQvDmjJ8SjWblpOJr
+
+Summary: http://api/v1/groups/grp/projects/prj/plans/1sMUTVjE25tQvDmjJ8SjWblpOJr
+2 👾 Pass (1 warning) arm64@gcc-11
+1 🎉 Pass arm64@gcc-10 👹 Fail: boot,ltp-smoke"""
         in result.output
     )
 
@@ -838,7 +819,7 @@ def test_test_cli(mocker, tuxsuite_config):
     assert test.call_count == 1
     test.assert_called_once_with(
         device="qemu-arm64",
-        kernel="",
+        kernel=None,
         modules=None,
         tests=["ltp-smoke"],
         boot_args=None,
@@ -950,6 +931,10 @@ def test_plan_cli_no_wait(mocker, tuxsuite_config, plan_config):
 
 
 sample_build_result = {
+    "project": "ci",
+    "git_repo": "https://example.com/linux.git",
+    "git_ref": "master",
+    "git_sha": "deadbeef",
     "build_name": "",
     "download_url": "https://builds.dev.tuxbuild.com/1sMUTVjE25tQvDmjJ8SjWblpOJr/",
     "errors_count": 0,
@@ -966,6 +951,7 @@ sample_build_result = {
 
 
 sample_test_result = {
+    "project": "ci",
     "device": "qemu-i386",
     "plan": "1sMNB0p8a3OrpUmEBgYah0Ycer9",
     "result": "fail",
@@ -973,6 +959,7 @@ sample_test_result = {
     "state": "finished",
     "tests": ["boot"],
     "uid": "1sMNBWLFrsOahyXQCNeiT4bgysP",
+    "waiting_for": None,
 }
 
 sample_all_result = {
@@ -1008,6 +995,16 @@ sample_all_result = {
 }
 
 
+sample_plan_result = {
+    "builds": {
+        "1": sample_build_result,
+        "2": {**sample_build_result, **dict({"warnings_count": 1})},
+        "3": {**sample_build_result, **dict({"warnings_count": 3})},
+        "4": {**sample_build_result, **dict({"result": "fail", "errors_count": 1})},
+    },
+    "tests": {"1": sample_test_result},
+}
+
 sample_plan_status = {
     "builds": {
         "1sewrBhxNVbsURAKBjeXX8pyjwY": sample_build_result,
@@ -1106,7 +1103,7 @@ def test_results_cli(
 
     mocker.resetall()
     results = mocker.patch(
-        "tuxsuite.Results.get_plan", return_value=(sample_all_result, "http://api")
+        "tuxsuite.Results.get_plan", return_value=(sample_plan_result, "http://api")
     )
     result = runner.invoke(
         tuxsuite.cli.results,
@@ -1116,7 +1113,17 @@ def test_results_cli(
     )
     assert result.exit_code == 0
     assert results.call_count == 1
-    assert format_result.call_count == 6
+    assert (
+        result.output
+        == """Summary: https://tuxapi.tuxsuite.com/v1/groups/tuxsuite/projects/tux/plans/1sMNB0p8a3OrpUmEBgYah0Ycer9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 🎉 Pass arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👾 Pass (1 warning) arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👾 Pass (3 warnings) arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👹 Fail (1 error) arm@gcc-9
+builds (4): 🎉 1 👾 2 👹 1
+tests (1): 👹 1
+"""
+    )
 
     mocker.resetall()
     results = mocker.patch(
@@ -1163,7 +1170,7 @@ def test_results_cli(
 
     mocker.resetall()
     results = mocker.patch(
-        "tuxsuite.Results.get_plan", return_value=(sample_all_result, "http://api")
+        "tuxsuite.Results.get_plan", return_value=(sample_plan_result, "http://api")
     )
     result = runner.invoke(
         tuxsuite.cli.results,
@@ -1173,7 +1180,17 @@ def test_results_cli(
     )
     assert result.exit_code == 0
     assert results.call_count == 1
-    assert format_result.call_count == 6
+    assert (
+        result.output
+        == """Summary: https://tuxapi.tuxsuite.com/v1/groups/tuxsuite/projects/tux/plans/null
+1sMUTVjE25tQvDmjJ8SjWblpOJr 🎉 Pass arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👾 Pass (1 warning) arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👾 Pass (3 warnings) arm@gcc-9
+1sMUTVjE25tQvDmjJ8SjWblpOJr 👹 Fail (1 error) arm@gcc-9
+builds (4): 🎉 1 👾 2 👹 1
+tests (1): 👹 1
+"""
+    )
 
 
 def test_results_cli_errors(mocker, tmp_path, tuxsuite_config, build_set_result):
@@ -1269,3 +1286,76 @@ def test_results_cli_errors(mocker, tmp_path, tuxsuite_config, build_set_result)
     )
     assert result.exit_code == 1
     assert results.call_count == 1
+
+
+def test_patch_series_error(mocker, tuxsuite_config):
+    runner = CliRunner()
+
+    # invalid url scheme
+    result = runner.invoke(
+        tuxsuite.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-sha=beefbee",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "--patch-series=httpsucks.patch",
+        ],
+    )
+    assert result.exit_code == 2
+    assert "Error: Invalid value for '-p' / '--patch-series'" in result.output
+
+    mocker.resetall()
+    # invalid patch file
+    result = runner.invoke(
+        tuxsuite.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-sha=beefbee",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "--patch-series=/tmp/invalidpatch",
+        ],
+    )
+    assert result.exit_code == 2
+    assert "Error: Invalid value for '-p' / '--patch-series'" in result.output
+
+
+def test_patch_series_pass(mocker, sample_patch_tgz, tuxsuite_config):
+    build = mocker.patch("tuxsuite.Build.build")
+    wait_for_object = mocker.patch("tuxsuite.cli.wait_for_object")
+    runner = CliRunner()
+    # valid url scheme
+    result = runner.invoke(
+        tuxsuite.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-sha=beefbee",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            "--patch-series=https://www.example.com/example.tgz",
+        ],
+    )
+    assert result.exit_code == 0
+    assert build.call_count == 1
+    assert wait_for_object.call_count == 1
+
+    mocker.resetall()
+    # valid patch file
+    result = runner.invoke(
+        tuxsuite.cli.build,
+        [
+            "--git-repo=https://git.example.com/linux.git",
+            "--git-sha=beefbee",
+            "--target-arch=arm64",
+            "--kconfig=defconfig",
+            "--toolchain=gcc-9",
+            f"--patch-series={sample_patch_tgz}",
+        ],
+    )
+    assert result.exit_code == 0
+    assert build.call_count == 1
+    assert wait_for_object.call_count == 1
