@@ -198,7 +198,7 @@ def test_submit(config, plan_config, mocker):
 
             for t, d in zip(tests, data):
                 assert t["device"] == d["device"]
-                assert t.get("kernel", "") == d.get("kernel")
+                assert t.get("kernel") == d.get("kernel")
                 assert t.get("tests") == d.get("tests")
 
             assert data[0]["waiting_for"] == "my-build-00"
