@@ -1,19 +1,23 @@
 # -*- coding: utf-8 -*-
 
 from attr import attrs, attrib
+import base64
 import concurrent.futures
 import json
 import queue
+import mimetypes
 import os
 import random
 import re
 import sys
+import tarfile
 import time
 import traceback
 import tuxsuite.exceptions
 import tuxsuite.requests
 from tuxsuite.version import __version__
 import uuid
+from urllib.parse import urlparse
 
 
 # We will poll for status change for an average duration of 180 minutes
@@ -50,6 +54,51 @@ def get_request(url, headers, params=None):
         response.raise_for_status()  # Some unexpected status that's not caught
 
 
+def handle_patch(patch_path):
+    patch_content = None
+    patch_file = os.path.basename(patch_path)
+    if urlparse(patch_path).scheme in ["http", "https"]:
+        patch_file = patch_path
+        patch_content = b""
+    elif os.path.isfile(patch_path):
+        file_type, file_encoding = mimetypes.guess_type(patch_path)
+        if file_type in [
+            "application/mbox",
+            "text/plain",
+            "text/x-diff",
+            "application/x-tar",
+        ]:
+            if file_encoding == "gzip":
+                series_exists = False
+                with tarfile.open(patch_path, "r") as patch:
+                    series_exists = "series" in [
+                        os.path.basename(f) for f in patch.getnames()
+                    ]
+                if not series_exists:
+                    raise Exception("series file missing in patch archive")
+            elif file_encoding is None:  # Valid for mbox format
+                pass
+            else:
+                raise Exception(
+                    f"Unsupported patch series archive format: {file_encoding}"
+                )
+        else:
+            raise Exception("Patch series format unknown")
+        with open(patch_path, "rb") as patch:
+            patch_content = base64.b64encode(patch.read())
+    else:
+        if not os.path.exists(os.path.join(patch_path, "series")):
+            raise Exception("series file not found in patch directory")
+        patch_path = os.path.abspath(patch_path)
+        patch_file = f"{os.path.basename(patch_path)}.tgz"
+        tar_file = os.path.join("/tmp", patch_file)
+        with tarfile.open(tar_file, "w:gz") as tar:
+            tar.add(patch_path, arcname=os.path.sep)
+        with open(tar_file, "rb") as patch:
+            patch_content = base64.b64encode(patch.read())
+    return (patch_file, patch_content.decode("utf-8"))
+
+
 class HeadersMixin:
     @property
     def headers(self):
@@ -73,7 +122,7 @@ class BuildState:
     final = attrib(default=False)
 
 
-@attrs(kw_only=True)
+@attrs(kw_only=True, order=False)
 class Build(HeadersMixin):
     git_repo = attrib()
     git_ref = attrib(default=None)
@@ -90,6 +139,7 @@ class Build(HeadersMixin):
     targets = attrib(default=[])
     make_variables = attrib(default={})
     kernel_image = attrib(default=None)
+    patch_series = attrib(default=None)
 
     build_data = attrib(default=None)
     build_name = attrib(default=None)
@@ -102,6 +152,9 @@ class Build(HeadersMixin):
         self.verify_build_parameters()
         self.client_token = str(uuid.uuid4())
 
+    def __lt__(self, other):
+        return (self.target_arch, self.toolchain) < (other.target_arch, other.toolchain)
+
     def __str__(self):
         git_short_log = self.status.get("git_short_log", "")
 
@@ -146,6 +199,11 @@ class Build(HeadersMixin):
             build_entry["build_name"] = self.build_name
         if plan is not None:
             build_entry["plan"] = plan
+        if self.patch_series:
+            (
+                build_entry["kernel_patch_file"],
+                build_entry["kernel_patch"],
+            ) = handle_patch(self.patch_series)
         return build_entry
 
     def verify_build_parameters(self):
@@ -162,10 +220,10 @@ class Build(HeadersMixin):
         assert self.toolchain, "toolchain is mandatory"
         assert self.headers is not None, "headers is mandatory"
 
-    def build(self):
+    def build(self, plan=None):
         """Submit the build request"""
         data = []
-        data.append(self.generate_build_request())
+        data.append(self.generate_build_request(plan=plan))
         url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
             self.group, self.project
         )
@@ -243,7 +301,8 @@ class Build(HeadersMixin):
                         icon="🔧",
                         cli_color="yellow",
                     )
-                    self.build()
+                    plan = self.status.get("plan", None)
+                    self.build(plan=plan)
                 elif state == "finished":
                     break
                 else:
@@ -495,7 +554,6 @@ class Plan(HeadersMixin):
                             group=self.group,
                             project=self.project,
                             device=test["device"],
-                            kernel="",
                             tests=test.get("tests", []),
                             boot_args=test.get("boot_args"),
                             wait_for=build.uid,
@@ -519,15 +577,16 @@ class Plan(HeadersMixin):
                         )
                     )
 
-        ret = post_request(
-            f"{self.url}/tests",
-            self.headers,
-            [t.generate_test_request(plan=self.plan) for t in tests],
-        )
-        for (test, data) in zip(tests, ret):
-            test.uid = data["uid"]
-            test.status = data
-            self.tests.append(test)
+        if tests:
+            ret = post_request(
+                f"{self.url}/tests",
+                self.headers,
+                [t.generate_test_request(plan=self.plan) for t in tests],
+            )
+            for (test, data) in zip(tests, ret):
+                test.uid = data["uid"]
+                test.status = data
+                self.tests.append(test)
 
         # Populate the status with builds and tests after submission
         for build in self.builds:
@@ -536,6 +595,44 @@ class Plan(HeadersMixin):
             tests_status.update({test.uid: test.status})
         self.status = {"builds": builds_status, "tests": tests_status}
 
+    def load(self, data):
+        for (uid, b) in data["builds"].items():
+            build = Build(
+                git_repo=b["git_repo"],
+                git_ref=b["git_ref"],
+                target_arch=b["target_arch"],
+                kconfig=b["kconfig"],
+                toolchain=b["toolchain"],
+                token=self.token,
+                kbapi_url=None,
+                tuxapi_url=self.tuxapi_url,
+                group=self.group,
+                project=b["project"],
+                environment=b.get("environment"),
+                targets=b.get("targets"),
+                make_variables=b.get("make_variables"),
+                kernel_image=b.get("kernel_image"),
+                build_data=b.get("build_data"),
+                build_name=b.get("build_name"),
+                status=b,
+                uid=b["uid"],
+            )
+            self.builds.append(build)
+
+        for (uid, t) in data["tests"].items():
+            test = Test(
+                device=t["device"],
+                token=self.token,
+                kbapi_url=None,
+                tuxapi_url=self.tuxapi_url,
+                group=self.group,
+                project=t["project"],
+                status=t,
+                tests=t["tests"],
+                uid=t["uid"],
+            )
+            self.tests.append(test)
+
     def get_plan(self):
         builds = {}
         tests = {}
@@ -559,6 +656,63 @@ class Plan(HeadersMixin):
         self.status = {"builds": builds, "tests": tests}
         return self.status
 
+    def _tests_wait_for(self, uid):
+        return [t for t in self.tests if t.status["waiting_for"] == uid]
+
+    def filter_builds(self, f):
+        return sorted([b for b in self.builds if f(self, b)])
+
+    def filter_tests(self, f):
+        return sorted([t for t in self.tests if f(self, t)])
+
+    def passing(self):
+        def __filter(plan, build):
+            return (
+                build.status["result"] == "pass"
+                and build.status.get("warnings_count", 0) == 0
+            ) and (
+                all(
+                    [
+                        t.status["result"] == "pass"
+                        for t in plan._tests_wait_for(build.uid)
+                    ]
+                )
+            )
+
+        return self.filter_builds(__filter)
+
+    def warning(self):
+        def __filter(plan, build):
+            return (
+                build.status["result"] == "pass"
+                and build.status.get("warnings_count", 0) != 0
+            ) and (
+                all(
+                    [
+                        t.status["result"] == "pass"
+                        for t in self._tests_wait_for(build.uid)
+                    ]
+                )
+            )
+
+        return self.filter_builds(__filter)
+
+    def failing(self):
+        def __filter(plan, build):
+            return build.status["result"] == "fail" or any(
+                [t.status["result"] == "fail" for t in self._tests_wait_for(build.uid)]
+            )
+
+        return self.filter_builds(__filter)
+
+    def errors(self):
+        def __filter(plan, build):
+            return build.status["result"] == "error" or any(
+                [t.status["result"] == "error" for t in self._tests_wait_for(build.uid)]
+            )
+
+        return self.filter_builds(__filter)
+
     def watch(self):
         n_builds = len(self.builds)
         n_tests = len(self.tests)
@@ -609,7 +763,7 @@ class Test(HeadersMixin):
     group = attrib()
     project = attrib()
     device = attrib()
-    kernel = attrib()
+    kernel = attrib(default=None)
     modules = attrib(default=None)
     tests = attrib(default=[])
     boot_args = attrib(default=None)
@@ -680,8 +834,8 @@ class Test(HeadersMixin):
 
         previous_state = None
         while True:
-            data = self.get()
-            state = data["state"]
+            self.status = self.get()
+            state = self.status["state"]
             if state != previous_state:
                 if state in waiting_states:
                     yield waiting_states[state]
@@ -698,7 +852,7 @@ class Test(HeadersMixin):
             previous_state = state
 
         errors = 0
-        result = data["result"]
+        result = self.status["result"]
         if result == "pass":
             icon = "🎉"
             message = "Pass"
@@ -707,7 +861,9 @@ class Test(HeadersMixin):
             icon = "👹"
             cli_color = "bright_red"
             errors = [
-                name for name in data["results"] if data["results"][name] == "fail"
+                name
+                for name in self.status["results"]
+                if self.status["results"][name] == "fail"
             ]
             message = "Fail ({})".format(", ".join(errors))
             errors = len(errors)
