@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import click
+from itertools import chain
 from functools import wraps
 import json
 import os
@@ -9,6 +10,7 @@ import tuxsuite
 import tuxsuite.download
 import tuxsuite.exceptions
 import tuxsuite.gitutils
+from urllib.parse import urlparse
 
 
 from tuxsuite.utils import (
@@ -78,7 +80,148 @@ def get_make_targets_vars(targets):
     return (target_list, make_variables)
 
 
-def format_result(result_json, tuxapi_tests_url=None):
+def plan_summary(plan):
+    bs = f"builds ({len(plan.builds)}):"
+    provisioning = len(
+        plan.filter_builds(lambda _, b: b.status["state"] == "provisioning")
+    )
+    running = len(plan.filter_builds(lambda _, b: b.status["state"] == "running"))
+    passing = len(
+        plan.filter_builds(
+            lambda _, b: b.status["result"] == "pass"
+            and b.status["warnings_count"] == 0
+        )
+    )
+    warning = len(
+        plan.filter_builds(
+            lambda _, b: b.status["result"] == "pass"
+            and b.status["warnings_count"] != 0
+        )
+    )
+    failing = len(plan.filter_builds(lambda _, b: b.status["result"] == "fail"))
+    error = len(plan.filter_builds(lambda _, b: b.status["result"] == "error"))
+
+    if provisioning:
+        bs += f" ⚙️  {provisioning}"
+    if running:
+        bs += f" 🚀 {running}"
+    if passing:
+        bs += f" 🎉 {passing}"
+    if warning:
+        bs += f" 👾 {warning}"
+    if failing:
+        bs += f" 👹 {failing}"
+    if error:
+        bs += f" 🔧 {error}"
+
+    ts = f"tests ({len(plan.tests)}):"
+    waiting = len(plan.filter_tests(lambda _, t: t.status["state"] == "waiting"))
+    provisioning = len(
+        plan.filter_tests(lambda _, t: t.status["state"] == "provisioning")
+    )
+    running = len(plan.filter_tests(lambda _, t: t.status["state"] == "running"))
+    passing = len(plan.filter_tests(lambda _, t: t.status["result"] == "pass"))
+    failing = len(plan.filter_tests(lambda _, t: t.status["result"] == "fail"))
+    error = len(plan.filter_tests(lambda _, t: t.status["result"] == "error"))
+
+    if waiting:
+        ts += f" ⏳ {waiting}"
+    if provisioning:
+        ts += f" ⚙️  {provisioning}"
+    if running:
+        ts += f" 🚀 {running}"
+    if passing:
+        ts += f" 🎉 {passing}"
+    if failing:
+        ts += f" 👹 {failing}"
+    if error:
+        ts += f" 🔧 {error}"
+    return (bs, ts)
+
+
+def format_plan_result(build, tests):
+    fail = False
+    if build.status["result"] == "pass":
+        if build.status["warnings_count"] == 0:
+            icon = "🎉"
+            message = "Pass"
+            cli_color = "green"
+        else:
+            icon = "👾"
+            cli_color = "yellow"
+            if build.status["warnings_count"] == 1:
+                message = "Pass (1 warning)"
+            else:
+                message = "Pass ({} warnings)".format(build.status["warnings_count"])
+    elif build.status["result"] == "fail":
+        fail = False
+        icon = "👹"
+        cli_color = "bright_red"
+        if build.status["errors_count"] == 1:
+            message = "Fail (1 error)"
+        else:
+            message = "Fail ({} errors)".format(build.status["errors_count"])
+    elif build.status["result"] == "error":
+        fail = False
+        icon = "🔧"
+        cli_color = "bright_red"
+        message = build.status["status_message"]
+    else:
+        raise NotImplementedError()
+
+    builds = (
+        build.uid
+        + " "
+        + click.style(f"{icon} {message}", fg=cli_color, bold=True)
+        + " "
+        + build.target_arch
+        + "@"
+        + build.toolchain
+    )
+
+    tests_str = ""
+    tests_pass = sorted(
+        set(
+            chain.from_iterable(
+                [t.tests for t in tests if t.status["result"] == "pass"]
+            )
+        )
+    )
+    tests_fail = sorted(
+        set(
+            chain.from_iterable(
+                [t.tests for t in tests if t.status["result"] == "fail"]
+            )
+        )
+    )
+    tests_error = sorted(
+        set(
+            chain.from_iterable(
+                [t.tests for t in tests if t.status["result"] == "error"]
+            )
+        )
+    )
+
+    if tests_pass:
+        tests_str += " 🎉 " + click.style(
+            f"Pass: {','.join(tests_pass)}", fg="green", bold=True
+        )
+    if tests_fail:
+        tests_str += " 👹 " + click.style(
+            f"Fail: {','.join(tests_fail)}", fg="bright_red", bold=True
+        )
+    if tests_error:
+        tests_str += " 🔧 " + click.style(
+            f"Error: {','.join(tests_error)}", fg="bright_red", bold=True
+        )
+
+    if fail or tests_fail or tests_error:
+        warning(builds + tests_str)
+    else:
+        info(builds + tests_str)
+
+
+def format_result(result_json, tuxapi_tests_url=None, prefix=""):
     state = result_states.get(result_json["state"], None)
     result = result_json["result"]
     result_msg = ""
@@ -141,7 +284,8 @@ def format_result(result_json, tuxapi_tests_url=None):
             errors=errors,
         )
     msg = (
-        click.style(f"{state.icon} {state.message}: ", fg=state.cli_color, bold=True)
+        prefix
+        + click.style(f"{state.icon} {state.message}: ", fg=state.cli_color, bold=True)
         + result_msg
     )
     if result == "fail" or result == "error":
@@ -150,6 +294,15 @@ def format_result(result_json, tuxapi_tests_url=None):
         info(msg)
 
 
+def file_or_url(path):
+    """Validate if path is a file/directory or an URL and check its existence"""
+    if urlparse(path).scheme in ["http", "https"]:
+        return path
+    elif os.path.exists(path):
+        return path
+    raise click.BadParameter(f"{path} does not exist or invalid")
+
+
 @click.group(name="tuxsuite")
 @click.version_option()  # Implement --version
 def cli():
@@ -242,6 +395,18 @@ def common_options(required):
             multiple=True,
             help="Set environment variables for the build. Format: KEY=VALUE",
         ),
+        option(
+            "-p",
+            "--patch-series",
+            type=file_or_url,
+            default=None,
+            help=(
+                "Patches to apply before building the kernel. Accepts patch "
+                "series that applies directly with 'git am' or "
+                "'git quiltimport' i.e., a mbox file or directory or gzipped "
+                "tarball (.tar.gz)"
+            ),
+        ),
     ]
 
     def wrapper(f):
@@ -302,6 +467,7 @@ def build(
     download=False,
     output_dir=None,
     no_wait=False,
+    patch_series=None,
     **build_params,
 ):
     quiet_output(quiet)
@@ -311,6 +477,9 @@ def build(
         build_params["targets"] = target_list
         build_params["make_variables"] = make_vars
 
+    if patch_series:
+        build_params["patch_series"] = click.format_filename(patch_series)
+
     try:
         build = tuxsuite.Build(**build_params)
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
@@ -361,6 +530,7 @@ def build_set(
     download=False,
     output_dir=None,
     no_wait=False,
+    patch_series=None,
     **build_params,
 ):
     quiet_output(quiet)
@@ -370,6 +540,9 @@ def build_set(
         build_params["targets"] = target_list
         build_params["make_variables"] = make_vars
 
+    if patch_series:
+        build_params["patch_series"] = click.format_filename(patch_series)
+
     try:
         build_set_config = tuxsuite.config.BuildSetConfig(set_name, tux_config)
         build_set = tuxsuite.BuildSet(build_set_config.entries, **build_params)
@@ -422,8 +595,11 @@ DEVICES = [
     "qemu-arm64",
     "qemu-i386",
     "qemu-mips",
+    "qemu-mips32",
+    "qemu-mips32el",
     "qemu-mips64",
     "qemu-mips64el",
+    "qemu-mipsel",
     "qemu-powerpc",
     "qemu-ppc64",
     "qemu-ppc64le",
@@ -436,6 +612,7 @@ TESTS = [
     "boot",
     "ltp-fcntl-locktests",
     "ltp-fs_bind",
+    "ltp-fs_perms_simple",
     "ltp-fsx",
     "ltp-nptl",
     "ltp-smoke",
@@ -444,7 +621,7 @@ TESTS = [
 
 @cli.command(help="Test a kernel", short_help="Test a kernel")
 @click.option("--device", help="Device type", required=True, type=click.Choice(DEVICES))
-@click.option("--kernel", help="URL of the kernel to test", default="", type=str)
+@click.option("--kernel", help="URL of the kernel to test", default=None, type=str)
 @click.option("--modules", help="URL of the kernel modules", default=None, type=str)
 @click.option(
     "--tests",
@@ -488,6 +665,8 @@ def test(device, kernel, modules, tests, boot_args, wait_for, no_wait, json_out)
                 "--modules and --wait-for are mutually exclusive"
             )
     else:
+        if not kernel:
+            raise click.ClickException("--kernel is required")
         info(
             "Testing {} on {} with {}".format(
                 kernel, device, ", ".join(["boot"] + tests)
@@ -625,13 +804,9 @@ def plan(
             format_result(test.status, plan.url + "/tests/{}".format(test.uid))
     else:
         result = wait_for_object(plan)
-        info(f"\nSummary: {plan.url}")
-        for build in plan.builds:
-            print_state(plan.results["builds"][build.uid], prefix="* ")
-            for test in [t for t in plan.tests if t.wait_for == build.uid]:
-                print_state(plan.results["tests"][test.uid], prefix="  -> ")
-        for test in [t for t in plan.tests if t.wait_for is None]:
-            print_state(plan.results["tests"][test.uid], prefix="* ")
+        info(f"\nSummary: {plan.url}/plans/{plan.plan}")
+        for b in chain(plan.passing(), plan.warning(), plan.failing(), plan.errors()):
+            format_plan_result(b, plan._tests_wait_for(b.uid))
 
     if json_out:
         json_out.write(json.dumps(plan.status, sort_keys=True, indent=4))
@@ -693,7 +868,7 @@ def results(build, test, plan, from_json, json_out):
                     if key == "plans":
                         info(f"{result['uid']}: {result['name']} {result['project']}")
                     else:
-                        format_result(result, tuxapi_tests_url)
+                        format_result(result, f"{tuxapi_tests_url}/{result['uid']}")
                 info("\n")
         if build:
             results.uid = build
@@ -705,14 +880,25 @@ def results(build, test, plan, from_json, json_out):
             format_result(result_json, tuxapi_tests_url)
         if plan:
             results.uid = plan
-            result_json, tuxapi_tests_url = results.get_plan()
-            for result in result_json["builds"].keys():
-                format_result(result_json["builds"][result])
-            for result in result_json["tests"].keys():
-                format_result(
-                    result_json["tests"][result],
-                    tuxapi_tests_url + "/{}".format(result),
-                )
+            result_json, tuxapi_plan_url = results.get_plan()
+
+            plan_obj = tuxsuite.Plan("")
+            plan_obj.plan = plan
+            plan_obj.load(result_json)
+            info(f"Summary: {plan_obj.url}/plans/{plan_obj.plan}")
+            for b in chain(
+                plan_obj.passing(),
+                plan_obj.warning(),
+                plan_obj.failing(),
+                plan_obj.errors(),
+            ):
+                format_plan_result(b, plan_obj._tests_wait_for(b.uid))
+            # TODO: print stand alone tests
+
+            (build_summary, test_summary) = plan_summary(plan_obj)
+            info(build_summary)
+            info(test_summary)
+
     except tuxsuite.exceptions.URLNotFound as e:
         raise (click.ClickException(str(e)))
 
